=begin
**
** dispatcher.rb
** 14/AUG/2007
** ETD-Software
**  - Daniel Martin Gomez <etd[-at-]nomejortu.com>
**
** Desc:
**   Dispathcer interface and requires to the different implementations.
**
** Version:
**  v1.0 [14/August/2007]: first released
**
** License:
**   See dradis.rb or LICENSE.txt for copyright and licensing information.
**
=end

module Commands

  # This module contains both the interface and the implementations for the
  # different dispatchers provided.
  #  
  # A dispatcher is an object that can handle user request and perform actions.
  # There is no limit in the amount or nature that a dispatcher can perform. 
  #
  # To create a dispatcher you only need to implement the 
  # DispatcherInterface. 
  #
  # Two implementations of the interface are provided:
  # - Simple: designed as the easiest way to implement the
  #   interface, classes inheriting from this dispatcher will only need to declare
  #   a list of *keywords* and provide the functions associated to them.
  # - Namespace: sometimes a single *keyword* is not enough
  #   to clearly identify an action. Say for example we want the user to be able
  #   to type: <tt>add host</tt> but also: <tt>add protocol</tt>. Since both
  #   actions are related to _adding_ something we want them to be handled by the
  #   same dispatcher, so we inherit from Namespace and set
  #   the :*namespace* setting to "+add+". Then we will need to implement the two
  #   methods (host, protocol) as if we were working with a Simple dispatcher.
  #  
  module Dispatchers
    
    # Provides the interface that dispatchers need to follow. The interface only 
    # requires from the dispatcher the ability to declare the actions that it 
    # supports DispatcherInterface#commands) and to be able to perform the action 
    # once the user has decided to do so DispatcherInterface#run.
    #
    # The details of the module should be provided in a Hash similar to this one:
    #  INFO = { 
    #    :commands => {
    #      'command #1' => {}, #options for the first command
    #      'command #2' => {}, #options for the second command
    #      #...
    #    }
    #  }
    #
    # The :*commands* key contains a Hash with all the information regarding the
    # different commands provided by the dispatcher. This keys for this hash
    # must be the command labels (+help+, +debug+, etc.) and the values must 
    # specify the options for each commands. For an example of this please see
    # DispatcherInterface#commands.
    #
    # Implementing classes could add further restrictions and features to the
    # above structure but the :*commands* key should be retained..
    module DispatcherInterface
      # Should return a Hash with the commands provided by the module. an example
      # of the syntax that the hash elements should follow is:
      # 
      #  'ls' => { 
      #    :desc => 'lists entries under the directory specified as argument', 
      #    :syntax => [ 
      #      {
      #        :required => true,
      #        :label=>'path', 
      #        :regexp=> /.+/
      #      }
      #    ]
      #  }
      #        
      # Because the framework will handle input validation, syntax checking,
      # help requests, etc. some information should be provided by the implementing
      # modules to do so. In a nutshell the description of the structure above:
      # [+ls+] is the name of the action. when the user enters +ls+ the method
      #        *ls* of the implementing module will be called.
      # [:*desc*] is the description of the nature of the action.
      # [:*syntax*] is an array of parameters accepted by the method. Each
      #             parameter is described as a Hash that contains three keys:
      #             [:*required*] states whether the parameter is required or
      #                           optional
      #             [:*label*] is the name of the parameter. This name will be
      #                        displayed if the user request help for the action
      #             [:*regexp*] the regular expresion that the parameter needs
      #                         to match
      def commands() raise "commands() must be overridden"; end
      
      #
      # Once the parser has decided the action that the user wants to perform,
      # this method is called.
      #
      # The first argument (_cmd_) is the action requested by the user. The
      # second is an array of the arguments provided by the user.
      #
      # No details for the implementation are provided here, but as a general
      # guideline this method should call to one of the methods of the module
      # depending on the value of _cmd_.
      def run(cmd, *args) raise "run(cmd, *args) must be overriden"; end
    end
  end
end

require 'commands/dispatchers/simple'
require 'commands/dispatchers/namespace'
