=begin
**
** add.rb
** 18/JUN/2007
** ETD-Software
**  - Daniel Martin Gomez <etd[-at-]nomejortu.com>
**
** Desc:
**   This module uses the SOAP interface to add information to the knowledge
** base of the dradis server.
**
** Version:
**  v1.0 [18/June/2007]: first released
**  v1.1 [24/October/2007]: return a formated output insted of a SOAP mapping object
**  v1.2 [05/February/2008]: new action: add note
**
** License:
**   See dradis.rb or LICENSE.txt for copyright and licensing information.
**
=end

module Commands
  
  # This module uses the SoapDriver to add information to the knowledge base of
  # the +dradis+ server. Actions provided are:
  # [+host+] add a new host
  # [+protocol+] add a new IP protocol
  # [+service+] add a new service for a host on a given port
  # [+category+] add a new note category
  # [+note+] add a new note to a specific host or service
  class Add < Dispatchers::Namespace
    include SoapDriver

 
    #----------------------------------------------------- private methods
    private
    INFO = {
      :namespace => 'add',
      :description => 'add new information to the knowledge base of the server',
      :commands => {
        'host' => {
          :desc => 'new host',
          :syntax => [
          {
            :required => true,
            :label=>'host address', 
            :regexp=> /\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}/
          }             
          ]
        },
        'protocol' => {
          :desc => 'new IP protocol',
          :syntax => [
          {
            :required => true,
            :label=>'protocol name', 
            :regexp=> /\w+/
          }             
          ]
        },
        'service' => {
          :desc => 'new service for a host on a given port',
          :syntax => [
          {
            :required => true,
            :label =>'host_name',  
            :regexp=> /\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}/
          },
          {
            :required => true,
            :label => 'service_name', 
            :regexp=> /\w+/
          },          
          {
            :required => true,
            :label => 'port', 
            :regexp=> /\d+/
          },          
          {
            :required => true,
            :label => 'protocol',
            :regexp=> /\w+/
          }              
          ]
        },
        'category' => {
          :desc => 'new note category',
          :syntax => [
          {
            :required => true,
            :label=>'category_name', 
            :regexp=> /\w+/
          }             
          ]
        },
        'note' => {
         :desc => 'new note to the given Host or Service',
          :syntax => [
          {
            :required => true,
            :label =>'type',  
            :regexp=> /Host|Service/
          },
          {
            :required => true,
            :label => 'id', 
            :regexp=> /\d+/
          },          
          {
            :required => true,
            :label => 'author', 
            :regexp=> /\w+/
          },          
          {
            :required => true,
            :label => 'category',
            :regexp=> /\w+/
          },
          {
            :required => true,
            :label  => 'text',
            :regexp => //
          }
          ]          
        }
      }
    }#:nodoc:

    def protocol(*args)
      if (args.size != 2)
        raise 'too few params'
      end
      setup_soap
      res = @driver.requestticket
      if res.code == 0
        setup_soap
        res = @driver.AddProtocol( {:ticket => res.value, :name => args[1]} )
      end
      return "#{res.value} (#{res.code})"
    end
  
    def host(*args)
      if ((args==nil) || (args.size != 2))
        raise 'too few params'
      end
      
      setup_soap
      res = @driver.requestticket
      if res.code == 0
        setup_soap
        res = @driver.AddHost( {:ticket => res.value, :address => args[1]} )
      end
      return "#{res.value} (#{res.code})"
    end

    def category(*args)
      if ((args==nil) || (args.size != 2))
        raise 'too few params'
      end
      
      setup_soap
      res = @driver.requestticket
      if res.code == 0
        setup_soap
        res = @driver.AddCategory( {:ticket => res.value, :name => args[1]} )
      end
      return "#{res.value} (#{res.code})"
    end

    #add service host_id service_name service_port service_protocol
    def service(*args)
      if ((args==nil) || (args.size != 5))
        raise 'too few params'
      end
      
      setup_soap
      res = @driver.requestticket
      if res.code == 0
        setup_soap
        res = @driver.AddService( 
        {
          :ticket => res.value, 
          :host => args[1],
          :name => args[2],
          :port => args[3],
          :protocol => args[4]
        }
        )
      end
      return "#{res.value} (#{res.code})"
    end
    
    def note(*args)
      if (args.size != 6)
        raise 'wrong number of arguments. please check your syntax.'
      end      
      setup_soap
      res = @driver.requestticket
      if res.code == 0
        setup_soap
        res = @driver.AddNote(
          {
          :ticket => res.value,
          :annotatable_type => args[1],
          :annotatable_id => args[2],
          :author => args[3],
          :category => args[4],
          :text=> args[5]
          }
        )
      end
      return "#{res.value} (#{res.code})"
    end

    #----------------------------------------------------- public methods
    public        

    
  end
end