=begin
**
** config.rb
** 27/SEP/2007
** ETD-Software
**  - Daniel Martin Gomez <etd[-at-]nomejortu.com>
**
** Desc:
**   Configuration parsing functionality. An interface with the XML parser
** implementation.
**
** Version:
**  v1.0 [27/September/2007]: first released
**
** License:
**   See dradis.rb or LICENSE.txt for copyright and licensing information.
**
=end

require 'rexml/document'

module Core

  # This module contains all the classes and interfaces needed to read and
  # parse Dradis configuration files.
  # A ParserInterface is provided just in case we want to change the
  # configuration container in the future. In this version we are using an
  # XML file, but a YAML or a database could be used provided that 
  # ParserInterface is followed.
  module Config
    #--------------------------------------------------- parser interface
    
    # The ParserInterface should be implemented by all the different
    # configuration parsers (xml, yaml, etc.). Dradis and it's modules will
    # only access methods defined in this interface
    module ParserInterface
      # Given an option name (as a symbol) this function retrieves the value
      # stored in the configuration file for it.
      def get_option(key)  
        raise 'unimplemented!'
      end
    end
    
    #--------------------------------------------------- parser implementations
    
    # ParserInterface implementation to read configuration parameters from a
    # XML file.
    class XMLParser
      include ParserInterface
      # Returns :*xml* symbol. This method is used by the Factory to choose the
      # appropiate implementation for a given file.
      def XMLParser.get_type() return :xml; end

      # Read and parse the XML file. Stores the options in an internal Hash to
      # speed up the retrieving process.
      def initialize(file)
        src = REXML::Document.new(File.new(file))
        @options = {}
        src.elements.each('dradis/option') do |element|
          @options[element.attributes['name'].to_sym] = element.attributes['value']
        end
      end
      
      # See ParserInterface#get_option
      def get_option(key)
        if @options.key?(key)
          @options[key]
        else
          raise ArgumentError.new('option not found in config file!')
        end
      end
    end
    
    #--------------------------------------------------- parser factory
    
    # Configuration Parser Factory class, used to get a valid configuration
    # parser for a given filename. The filename's extension is used to 
    # instantiate a valid ParserInterface implementation.
    class Factory
      @@factories = [XMLParser]
      
      # Given a configuration filename, this function returns a configuration
      # parser object that is able to read the values from the file.
      def Factory.from(file)
        type = File.extname(file).to_sym
        
        @@factories.each do |parser|
          parser.get_type() == type 
          return parser.new(file)
        end 
      end
    end
  end
end