=begin
**
** controller.rb
** 03/FEB/2008
** ETD-Software
**  - Daniel Martin Gomez <etd[-at-]nomejortu.com>
**
** Desc:
**   One of the important pieces of the architecture, this file contains the 
** implementation of the Controller interface. The Controller connects the
** View to the Service layer.
**
** Version:
**  v1.0 [03/February/2008]: first released
**  v1.1 [05/February/2008]: implement 'parse_command' as required by the 
**            Controller interface.
**  v1.2 [06/February/2008]: capture the Core::DradisQuitApp exception, invoke
**            the teardown() method of the view.
**  v1.3 [20/February/2008]: implement find_note.
**  v1.4 [31/March/2008|siebert]: implement a command line parser that is able
**            to accept multiple words in an argument if they are enclosed with
**            single or double quotes.
**
** License:
**   See dradis.rb or LICENSE.txt for copyright and licensing information.
**
=end

module Core
  # Context is the provided implementation of the Controller interface.
  class Controller
    include Core::ControllerInterface
    
    #===================================================== public methods
    public
    
    def initialize(params)
      
      # init the View
      raise ArgumentError.new('You need to define a :view') unless params.key?(:view)
      @view = params.fetch(:view)
      @view.setup(self)
      
      # init the Service
      @service = Service.new
    end
       
    def exec() 
      @view.run
    end
    
    def invoke(cmd,*args)
      # do input validation on cmd agains the list of available services
      begin
        output = @service.dispatch(cmd, *args)
      rescue Errno::ECONNREFUSED
        output = "Error while connecting. Is the server running?"
        $logger.error{$!.to_s}
      rescue Core::DradisQuitApp
        output = 'Closing dradis.'
        @view.teardown()
      rescue
        output = "[error] #{$!}"
        $logger.error {$@.join("\n\t")}
        #@base.textConsole.append "error]\t" + $!
        #@base.textConsole.append $!.backtrace.sort.join("\n\t")
        #@base.textConsole.append $@.join("\n\t")
      end
      return output
    end
    
    def parse_command(line)
      
      # parse the commandline taking quotes and double quotes in account
      args = line.split
      # will keep args after parsing
      final_args = []
      counter = 0      
      # loop through whole array
      while counter < args.length
        for i1 in counter..args.length-1
          final_args << args[i1]
          counter = counter + 1
          # we see if element starts with " or '
          if args[i1][0,1] == '"' or args[i1][0,1] == '\''
            # value remember if we found a " or a '
            value = args[i1][0,1]
            # lets just check that this argument does not end in a quote
            if args[i1][-1,1] == value
              final_args << final_args.pop[1...-1]
              break
            else
              final_args << final_args.pop[1..-1]
            end
            # now loop through the subsequent elements
            for i2 in i1+1..args.length-1
              # pad with space
              final_args.last << ' '
              # append element
              final_args.last << args[i2]
              counter = counter + 1
              # check if element ends with +value+
              if args[i2][-1,1] == value
                # amend and break if this is the case
                final_args << final_args.pop[0...-1]
                break
              end
            end
            break
          end
        end

      end
      
      self.invoke(final_args[0], *final_args)
      #self.invoke(line.split.shift, *line.split)
    end
    
    def model()
      @service.model()
    end
    
    # TODO: not sure if this is the best way for searching notes
    def notes_for(annotatable_type, annotatable_id) 
      notes = []
      
      case annotatable_type
      when :host then
        @controller.model.hosts.each do |host|
          next unless host.id == annotatable_id
          notes = host.notes
          break
        end
      when :service then
        found = false
        @controller.model.hosts.each do |host|
          host.services.each do |service|
            next unless service.id = annotatable_id
            notes = service.notes
            found = true
            break
          end
          break if found
        end
      end      
      
      notes
    end
    
    # TODO: no good
    def add_note(annotatable_type,annotatable_id,author,category,text)
      type = (annotatable_type == :host) ? 'Host' : 'Service'
      self.invoke('add', 'add', 'note', type, "#{annotatable_id}",author,category,text )
    end  
    def edit_note(note_id,author,category,text)
      self.invoke('edit', 'edit', 'note', "#{note_id}",author,category,text )
    end  
    
    # Given a _note_id_ this method searches the model and returns the Note
    # object associated with it.
    def find_note(note_id)
      note = nil
          
      @service.model.hosts.each do |host|
        host.notes.each do |n|
          next unless n.uid == note_id
          note = n
        end
        host.services.each do |srv|
          srv.notes.each do |n|
            next unless n.uid == note_id
            note = n
          end
        end
      end
      return note
    end 
      
    def refresh_model()
      self.invoke('refresh', 'refresh', "#{self.model.revision}")
    end
  end

end