# = Synopsys
#
# This module is the main entry point of the application. Within it, the user
# interface is instanced and launched.
#
# = Author
# 
# Daniel Martin Gomez, MWR Infosecurity, UK.
# <daniel.martin-gomez[-at-]mwrinfosecurity.com>, <etd[-at-]nomejortu.com>
#
# = Usage
# 
#   ruby dradis.rb [ -h | -help ] [ -c | -g ]
#   
#   -h | --help: display application synopsys, author and usage information.
#   -c: choose comand line interface (default)
#   -w: choose a graphical interface based on WxWidgets (requires wxruby gem)
#   -q: choose a graphical interface based on Qt4 (requires libqt4-ruby bindings)
#   -g: start graphical user interface (same as -w)
# 
# = License
# 
# This file may be used under the terms of the GNU General Public
# License version 2.0 as published by the Free Software Foundation
# and appearing in the file LICENSE.txt included in the packaging of
# this file.
#
# Copyright: Daniel Martin Gomez <etd[-at-]nomejortu.com>
#

# = Version
#  v1.0 [21/Jun/2007]: first released
#  v1.1 [01/Feb/2008]: add the new wxWidgets library as default graphical 
#    interface
#  v1.2 [05/February/2008]: create a new command line option to launch the Qt
#    interface, and another to launch the WxWidgets. The old -g defaults to 
#    WxWidgets at the moment
#  v1.3 [06/February/2008]: 
#    - modify the syntax information to include the new -w and -q options
#    - include a begin/rescue block to spot problems when loading the different
#    graphical interfaces.

require 'core/config'
require 'core/exception'
require 'core/context'

require 'optparse'
require 'rdoc/usage'
require 'logger'

if $0 == __FILE__
  ui = :cli
  
  # TODO: add command line option to launch Qt, Wx. Use -g for the default
  # interface for the underlaying platform :)
  opts = OptionParser.new
  opts.on('-h', '--help') {RDoc::usage}
  opts.on('-c') {ui = :cli}
  opts.on('-g') {ui = :wx}
  opts.on('-q') {ui = :qt}
  opts.on('-w') {ui = :wx}
  opts.parse(ARGV) rescue RDoc::usage(0, 'usage')

  $logger = Logger.new(STDOUT)
  $logger.level = Logger::DEBUG
  #$logger.level = Logger::FATAL
  $logger.datetime_format = '%H:%M:%S'
  
  $config = Core::Config::Factory.from('conf/dradis.xml')

  begin
    case ui
    when :qt #Qt application
      $logger.debug{'starting Qt interface'}
      require 'ui/qt/gui'
      ui = Ui::Qt4::DradisGui.new(ARGV)
    when :wx # wXWidgets application
      $logger.debug{'starting wxWidgets interface'}
      require 'ui/wx'
      ui = Ui::WxWidgets::DradisGui.new
    when :cli #Console application
      $logger.debug{'starting command line interface'}
      require 'ui/console'
      ui = Ui::Console::CLI.new()
    end
    controller = Core::Controller.new(:view => ui)
    controller.exec
  
  rescue LoadError
    $logger.error('Please try an alternative client interface. Use -h for options')
  ensure
    $logger.close  
  end
end
