=begin
**
** window.rb
** 01/FEB/2008
** ETD-Software
**  - Daniel Martin Gomez <etd[-at-]nomejortu.com>
**
** Desc:
**  Main wxWidgets application window
**
** Version:
**  v1.0 [01/February/2008]: first released
**  v1.1 [05/February/2008]: take advantage of the new 'parse_command' method
**             of the ControllerInterface
**  v1.2 [19/February/2008]: 
**    - get rid of the unused create_html_ctrl method
**    - add handlers for the events thrown by the NotesBrowser control.
**    - require the newly created dialogs.rb that contains GUI dialogs required
**  v1.3 [20/February/2008]: 
**    - use standard Wx::ID_EXIT for the menu item (adds a nice icon).
**    - implement on_add_note and on_edit_note
**  v1.4 [29/February/2008]: 
**    - get rid of the create_html_ctrl method.
**    - implement the 'toggling' functionality of the items of the 'View' menu.
**    whenever a pane is closed, the menu item associated unticks. When the user
**    activates the menu, the pane is show again.
**  v1.5 [03/April/2008]: fix the window.rb:159 bug. Set @prev_selection to nil
**    when the tree needs to be refreshed.
**
** License:
**   See dradis.rb or LICENSE.txt for copyright and licensing information.
**
=end

require 'ui/wx/widgets'
require 'ui/wx/dialogs'

module Ui
  module WxWidgets

    class  DradisWindow < Wx::Frame
      consts = %w[
        ID_FileExit
        ID_ViewConsole
        ID_ViewHosts
        ID_TimerUpdates
      ]
      consts.each_with_index do | c, i |
        const_set(c, 2000 + i) 
      end
      attr_writer :controller
      attr_reader :consoleFrame
      
      #===================================================== private methods
      private      

      #----------------------------------------------------- setup the interface
      def setup_menu
        menu_bar = Wx::MenuBar.new

        file_menu = Wx::Menu.new
        file_menu.append(Wx::ID_EXIT, 'Exit')
        
        view_menu = Wx::Menu.new
        view_menu.append_check_item(ID_ViewConsole, 'Console')
        view_menu.append_check_item( ID_ViewHosts, 'Hosts')

        menu_bar.append(file_menu, 'File')
        menu_bar.append(view_menu, 'View')
        
        menu_bar.check(ID_ViewConsole, true)
        menu_bar.check(ID_ViewHosts, true)
        set_menu_bar(menu_bar)
      end

      def setup_events
        # associate menus with their action methods
        evt_menu(Wx::ID_EXIT) { |e| on_exit }
        evt_menu(ID_ViewConsole) { |e| on_toggle_pane(:console)}
        evt_menu(ID_ViewHosts) { |e| on_toggle_pane(:hosts)}
        
        # advanced user interface
        evt_aui_pane_close { |e| on_pane_close(e) }
        
        # custom control event handling
        evt_new_command(@console.get_id) { |e| on_new_command(e) }
        evt_tree_sel_changed(@hosts.get_id) { |e| on_tree_clicked(e) }
        evt_add_note(@notes) { |e| on_add_note(e) }
        evt_edit_note(@notes) { |e| on_edit_note(e) }
        
        # setup periodic update timer
        @timer_updates = Wx::Timer.new(self, ID_TimerUpdates)
        # TODO: modify :qt_refresh to :ui_refresh
        @timer_updates.start( $config.get_option(:qt_refresh).to_i )
        evt_timer(ID_TimerUpdates) { |e| on_timeout_updates }
      end

      def setup_panes
        
        @console = Widgets::Console.new(self)
        pi_c = Wx::AuiPaneInfo.new
        pi_c.set_name('console_pane').set_caption('Console')
        pi_c.bottom.set_layer(1).set_position(1)
        @manager.add_pane(@console, pi_c)

        @hosts = Widgets::HostsBrowser.new(self)
        pi_h = Wx::AuiPaneInfo.new
        pi_h.set_name('hosts_pane').set_caption('Hosts')
        pi_h.left.set_layer(1).set_position(1)
        @manager.add_pane(@hosts, pi_h)

        @notes = Widgets::NotesBrowser.new(self)
        pi_n = Wx::AuiPaneInfo.new
        pi_n.set_name('html_content').center_pane.hide
        @manager.add_pane(@notes, pi_n)


        @manager.get_pane('amigo').show.bottom.set_layer(0).set_row(0).set_position(0)
        @manager.get_pane('html_content').show
      end
      
      #----------------------------------------------------- menu actions
      def on_exit
        close(true)
      end
      
      def on_toggle_pane(panel_key)
        pane = nil
        case panel_key
        when :console
          pane = @manager.get_pane(@console)
        when :hosts
          pane = @manager.get_pane(@hosts)
        end
        pane.is_shown ? pane.hide : pane.show
        @manager.update
      end
      
      #----------------------------------------------------- advanced user interface
      def on_pane_close(event)
        id = nil
        case event.get_pane.get_name
        when 'console_pane'
          id = ID_ViewConsole
        when 'hosts_pane'
          id = ID_ViewHosts
        end
        get_menu_bar.find_item(id).check(false)
      end
      
      
      #----------------------------------------------------- custom controls
      def on_new_command(event)
        line = event.line
        output = @controller.parse_command(line)
        @console.append(output.to_s)        
      end
      def on_tree_clicked(event)
        if event.get_item.nonzero?
          item_id = event.get_item
          @hosts.set_item_bold(@prev_selection, false) unless @prev_selection.nil?
          @hosts.set_item_bold(item_id)
          @prev_selection = item_id
          
          data = @hosts.get_item_data(item_id)
          @notes.show_notes(data[:type], data[:id], data[:notes])
        end 
      end
      def on_add_note(event)
        dialog = Dialogs::NoteDialog.new
        dialog.categories = @controller.model.categories
        if dialog.show_modal == Wx::ID_OK
          @controller.add_note(
            event.annotatable_type, 
            event.annotatable_id, 
            dialog.author, 
            dialog.category, 
            dialog.text)
          @notes.invalidate!
        end
        dialog.destroy
      end
      def on_edit_note(event)
        note = @controller.find_note(event.note_uid)
        
        dialog = Dialogs::NoteDialog.new
        dialog.categories = @controller.model.categories
        dialog.edit_note(note)
        
        if dialog.show_modal == Wx::ID_OK
          if (( dialog.author != note.author ) |
            ( dialog.category != note.category ) |
            ( dialog.text != note.text )
            )
            @controller.edit_note(
              event.note_uid, 
              dialog.author, 
              dialog.category, 
              dialog.text)
          
            @notes.invalidate!
          end          
        end
        dialog.destroy
      end
      #----------------------------------------------------- other events
      def on_timeout_updates
        old_revision = @controller.model.revision
        out = @controller.invoke('refresh', 'refresh', "#{old_revision}")
        @console.append( out ) unless (out.nil?)
        if (old_revision != @controller.model.revision)
          # update the view
          update_view
        end
      end
      
      def update_view
        @prev_selection = nil
        @hosts.update_view(@controller.model)
        @notes.home
      end
      #===================================================== public methods
      public
      def initialize(*args)
        super
        @manager = Wx::AuiManager.new
        @manager.set_managed_window(self)
        setup_menu()
        #create_status_bar()
        set_min_size( Wx::Size.new(400,300) )
        setup_panes()
        setup_events()
        @manager.update()
      end            
    end
  end
end
